(*------------------------------------------------------------------------------
  TOleRegister v 1.2   2/17/2000
  1999 Jonathon Smith, N-Sync Systems
  Tested with Delphi 5, Cbuilder 3&4 under Win98, WinNT 4.0 & Win2000

  History: v 1.2 : Changed all strings to AnsiStrings for compatibility with Cbuilder
           v 1.0 : (what happened to 1.1? - don't ask) Initial coding.

  Purpose: Facilitate a simple way to register and unregister common OLE
           objects. Didn't see a need for a file dialog on the filename
           as I intend on using it for installations... do what YOU gotta do...
           No rights reserved as I was given code chunks.

  Usage:   set the FileName property to the OLE object, call RegisterOLE or
           UnregisterOLE to do the deed.

  Credits: To the various (un<known|remembered>) sources for the pieces...
           Anyone who has ever helped or given away a component.
           The Delphi community is without equal anywhere!
------------------------------------------------------------------------------*)
unit OleRegister;

interface

uses
    Windows, Classes, SysUtils, ComObj, ComCtrls, StdCtrls, Registry,
    ActiveX, ShellAPI;

type
  TRegisterMode = (modeRegister, modeUnRegister);

  TOleRegister = class(TComponent)
  private
    { Private declarations }
    FFileName : AnsiString;
    function OleRegDLL(Mode : TRegisterMode) : Boolean;
    function OleRegTLB(Mode : TRegisterMode) : Boolean;
    function OleRegEXE(Mode : TRegisterMode) : Boolean;
    function OleReg(Mode : TRegisterMode) : Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;
    function RegisterOLE : boolean;
    function UnRegisterOLE : boolean;
  published
    { Published declarations }
    property FileName : AnsiString read FFileName write FFileName;

  end;

//-------------------------------

procedure Register;

implementation

function TOleRegister.OleRegDLL(Mode : TRegisterMode) : Boolean;
type
    funcOleRegister = function : HResult;
var
   hLib  : THandle;
   fnAdr : TFarProc;
begin
  Result := False;
  hLib := LoadLibrary(PChar(FFileName));
  if (hLib > 0) then
  begin
   try
     if (Mode = modeRegister) then
      fnAdr := GetProcAddress(hLib, PChar('DllRegisterServer'))
     else
      fnAdr := GetProcAddress(hLib, PChar('DllUnregisterServer'));
     if (fnAdr <> nil) then
       Result := (funcOleRegister(fnAdr) = 0);
   finally
     FreeLibrary(hLib);
   end;
 end;
end;

function TOleRegister.OleRegTLB(Mode : TRegisterMode) : Boolean;
type
  TUnRegTlbProc = function (const libID: TGUID; wVerMajor, wVerMinor: Word;
  lcid: TLCID; syskind: TSysKind): HResult; stdcall;
var
   wstrFileName   : WideString;
   wstrDocName    : WideString;
   iTypeLibrary   : ITypeLib;
   pLibAttr       : PTLibAttr;
   hOleAutLib     : THandle;
   fnUnRegTlbProc : TUnRegTlbProc;
begin
   Result := True;
   try
     wstrFileName := FFileName;
     OLECheck(LoadTypeLib(PWideChar(wstrFileName), iTypeLibrary));
     OLECheck(iTypeLibrary.GetLibAttr(pLibAttr));
     try
       if (Mode = modeRegister) then
       begin
         OleCheck(iTypeLibrary.GetDocumentation(-1, nil, nil, nil, @wstrDocName));
         wstrDocName := ExtractFilePath(wstrDocName);
         OleCheck(RegisterTypeLib(iTypeLibrary, PWideChar(wstrFileName ), PWideChar(wstrDocName)));
       end
       else
       begin
         hOleAutLib := GetModuleHandle('OLEAUT32.DLL');
         if (hOleAutLib <> 0) then
         begin
          @fnUnRegTlbProc := GetProcAddress(hOleAutLib, 'UnRegisterTypeLib');
          OleCheck(fnUnRegTlbProc(pLibAttr^.Guid,pLibAttr^.wMajorVerNum,pLibAttr^.wMinorVerNum,pLibAttr^.LCID,pLibAttr^.SysKind));
         end; // inner if
       end;   // else
       finally
         iTypeLibrary.ReleaseTLibAttr(pLibAttr);
       end;  // try-finally
     except
       Result := False;
     end; // try-except
end;

function TOleRegister.OleRegEXE(Mode : TRegisterMode) : Boolean;
begin
  if (Mode = modeRegister) then
    Result := (ShellExecute(0, 'open', PChar(FFileName), '/RegServer', Nil, SW_SHOWNORMAL) > 32)
  else
    Result := (ShellExecute(0, 'open', PChar(FFileName), '/UnregServer', Nil, SW_SHOWNORMAL) > 32);
end;

function TOleRegister.OleReg(Mode : TRegisterMode) : Boolean;
var
   strExt : AnsiString;
begin
  Result := FALSE;
  if (FileExists(FFileName)) then
  begin
    strExt := ExtractFileExt(FFileName);
    if (strExt <> '') then
    begin
      strExt := Copy(strExt, 2, Length(strExt));
      if(CompareText (strExt, 'OCX') = 0)
         or
        (CompareText (strExt, 'DLL') = 0)
        then
        Result := OleRegDLL(mode)
      else
        if (CompareText (strExt, 'TLB') = 0) then
          Result := OleRegTLB(mode)
        else
        if (CompareText (strExt, 'EXE') = 0) then
          Result := OleRegEXE(mode)
    end;
  end;
end;

// Yet another level of indirection - all so I can use properties!

function TOleRegister.RegisterOLE : boolean;
begin
  Result := OleReg(modeRegister);
end;

function TOleRegister.UnRegisterOLE : boolean;
begin
  Result := OleReg(modeUnRegister);
end;

(*********** H O U S E  K E E P I N G *******************)

constructor TOleRegister.Create(AOwner: TComponent);
begin
inherited Create(AOwner);
  FFileName := '';
end;

destructor TOleRegister.Destroy;
begin
  inherited Destroy;
end;

{----------- R E G I S T E R ------------------}

procedure Register;
begin
  RegisterComponents('NSync', [TOleRegister]);
end;

end.

